#!/usr/bin/env perl

our $VERSION = '0.21';

use warnings;
use strict;
use Pod::Usage;
use Getopt::Long qw(GetOptionsFromArray :config gnu_compat);

use Exif;
use Exposure;

my $help = 0;
my $man  = 0;
my $versions;

my %options = (
	'help'     => \$help,
	'man'      => \$man,
	'versions' => \$versions,
	'info'     => 0,
	'output'   => '',
);

GetOptions( \%options, 'help', 'man', 'versions', 'info|i', 'output|o=s', )
  or pod2usage( -verbose => 0 ) && exit 1;

if ($versions) {
	print
	  "\nModules, Perl, OS, Program info:\n",
	  "  Pod::Usage            $Pod::Usage::VERSION\n",
	  "  Getopt::Long          $Getopt::Long::VERSION\n",
	  "  strict                $strict::VERSION\n",
	  "  warnings              $warnings::VERSION\n",
	  "  Perl                  $]\n",
	  "  OS                    $^O\n",
	  "  jpgfixexif            $VERSION\n",
	  "  Exif                  $Exif::VERSION\n",
	  "  Exposure              $Exposure::VERSION\n\n";
	exit;
}

pod2usage( -verbose => 1 ) && exit if $help;
pod2usage( -verbose => 2 ) && exit if $man;
pod2usage("$0: Need one JPG digital photograph.\n")
  && exit
  unless ( $#ARGV == 0 );
$options{input} = $ARGV[0];    # JPEG image

jpgfixexif(%options) unless caller();

sub jpgfixexif {
	my %opts = @_;

	die("Can't find file $opts{input}") unless ( -f $opts{input} );

	my $exif     = &get_exif_tags( $opts{input} );
	my %settings = &get_settings($exif);
	die("Can't determine aperture")      unless $settings{aperture};
	die("Can't determine shutter speed") unless $settings{shutter};
	die("Can't determin film speed")     unless $settings{iso};
	my $exposure = &exposure_from_settings(%settings);

	if ( $opts{info} ) {
		print <<EndOfInfo;
ExposureTime : $settings{shutter}
FNumber      : $settings{aperture}
ISO          : $settings{iso}
=> Exposure  : $exposure
Dry-run only, no changes were made.
EndOfInfo
		return;
	}

	my $fixed = new Image::ExifTool;
	$fixed->SetNewValue( ISO          => $settings{iso} );
	$fixed->SetNewValue( FNumber      => $settings{aperture} );
	$fixed->SetNewValue( ExposureTime => $settings{shutter} );

	my $input = $opts{input};
	$fixed->WriteInfo( "$input", "$input.new" );

	# default is to append '_original', like exiftool does.
	if ( !$opts{output} ) {
		rename "$input",     "${input}_original" or
				die("Cannot rename $input to ${input}_original: $?");
		rename "$input.new", "$input" or
				die("Cannot rename $input.new to $input: $!");
	} else {
		rename "$input.new", $opts{output} or
				die("Cannot rename $input.new to $opts{output}: $!");
	}
}

__END__

=head1 NAME

B<webhdr_jpgfixexif> - Fix exposure info in EXIF header.

=head1 SYNOPSIS

webhdr_jpgfixexif [options] photograph.jpg

Options:
     --help        brief help message
    -m|--man       full documentation
     --versions    print version info of loaded modules
    -o|--output    output file name.
    -i|--info      only print exposure information, but make no changes

=head1 DESCRIPTION

In order to determine the exposure value of an image, most HDR software
relies upon the EXIF information that is embedded within a digital photograph.
This works fairly reliably in many cases. However, the implementation of
the EXIF specification is not consistent across camera manufacturers
and models.

B<webhdr_jpgfixexif> will attempt to fix the EXIF information in a digital
photograph so that the values 'ISO', 'FNumber', and 'ExposureTime' are
present.

=head1 OPTIONS

=over 8

=item B<--help>

Print a brief help message and exit.

=item B<-m|--man>

Print the manual page and exit.

=item B<--versions>

Print version information of the program, loaded modules, Perl, and the OS.

=item B<-o|--output>

Output file.  The default is to rename 'input.jpg' to 'input.jpg_original',
and to save to 'input.jpg'. This is consistent with exiftool, which
B<webhdr_jpgfixexif> relies upon.

=item B<-i|--info>

Print information for ISO, FNumber, and ExposureTime, as well as the
exposure value that can be derived from those values.  Exit without making
any changes.

=back

=head1 EXAMPLE

The check whether B<webhdr_jpgfixexif> works with our photograph:

  webhdr_jpgfixexif --info DSCN2386.JPG

To fix the EXIF header in DSCN2386.JPG and save to DSCN2386_fixed.jpg:

  webhdr_jpgfixexif --output DSCN2386_fixed.jpg DSCN2386.JPG

=head1 PREREQUISITES

Phil Harvey's ExifTool (http://www.sno.phy.queensu.ca/~phil/exiftool/)

=head1 ENVIRONMENT

webhdrtools's lib directory needs to be in PERL5LIB

=head1 COPYRIGHT

Copyright 2013 Axel Jacobs

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

#######################################################################
# ChangeLog
# ---------
#
# 0.21, 02 Jun 2013: Minor typos
# 0.2,   05 Aug 2012: Update POD;
# 0.1,   02 Apr 2011: Initial version
#######################################################################

#EOF
