#!/usr/bin/perl

our $VERSION = '0.27';

use strict;
use warnings;
use Image::Magick;
use Pod::Usage;
use Getopt::Long;

use ImageLDR;

my $debug    = 0;
my $help     = 0;
my $man      = 0;
my $versions = 0;

my %options = (
	'debug'    => 0,
	'help'     => \$help,
	'man'      => \$man,
	'versions' => \$versions,
	'dim'      => 512,
	'output'   => 'heatmap.png',
	'upper'    => 228,
	'lower'    => 27,
);

GetOptions(
	\%options,  'debug',   'help',       'man',
	'versions', 'dim|d=i', 'output|o=s', 'upper|u=i',
	'lower|l=f',
) or pod2usage( -verbose => 0 ) && exit 1;

if ($versions) {
	print
	  "\nModules, Perl, OS, Program info:\n",
	  "  Pod::Usage            $Pod::Usage::VERSION\n",
	  "  Getopt::Long          $Getopt::Long::VERSION\n",
	  "  Image::Magick         $Image::Magick::VERSION\n",
	  "  strict                $strict::VERSION\n",
	  "  warnings              $warnings::VERSION\n",
	  "  Perl                  $]\n",
	  "  OS                    $^O\n",
	  "  jpgheatmap            $VERSION\n",
	  "  ImageLDR              $ImageLDR::VERSION\n\n";
	exit 0;
}

pod2usage( -verbose => 1 ) && exit if $help;
pod2usage( -verbose => 2 ) && exit if $man;
pod2usage("$0: No files given.\n") && exit
  unless ( ( $#ARGV >= 0 ) or caller() );
foreach (@ARGV) {
	push( @{ $options{inputs} }, $_ );    # JPEG images
}

jpgheatmap(%options) unless caller();

sub jpgheatmap {
	my %opts = @_;

	die("Upper threshold must be between 100 and 255.")
	  unless ( ( $opts{upper} > 100 ) and ( $opts{upper} < 255 ) );
	die("Lower threshold must be between zero and 100.")
	  unless ( ( $opts{lower} > 0 ) and ( $opts{lower} < 100 ) );

	# Convert upper and lower threshold from 0..255 range to percentages
	$opts{upper} /= 2.55;
	$opts{lower} /= 2.55;

	my @sequence = ();
	foreach ( @{ $options{inputs} } ) {
		die("Can't find file $_\n") unless ( -f "$_" );
		push( @sequence, "$_" );
		my $avrg = &get_average("$_");
	}

	my ( $sorted_ref, $brights_ref ) = &sort_brightness(@sequence);
	my @sorted  = @$sorted_ref;              # file names
	my @brights = @$brights_ref;             # brightness in 0..1 range
	my $idx     = &get_greyest_idx(@brights);
	my $median  = $sorted[$idx];             # image that is closest to 50% grey

	# Generate blue pixels from brightest image
	my $bmask = Image::Magick->new;
	my $err   = $bmask->Read("$sorted[0]");

	$bmask->Threshold( 'threshold' => "$opts{lower}%", 'channel' => 'Red' );
	$bmask->Threshold( 'threshold' => "$opts{lower}%", 'channel' => 'Green' );
	$bmask->Threshold( 'threshold' => "$opts{lower}%", 'channel' => 'Blue' );
	$opts{debug} && $bmask->Write('brightmask0.png');
	$bmask->Threshold( 'threshold' => '99%' );
	$opts{debug} && $bmask->Write('brightmask1.png');

	$bmask->Resize(
		'geometry' => "$opts{dim}x$opts{dim}",
		'filter'   => 'cubic'
	);
	$bmask->Threshold( 'threshold' => '99%' );
	$bmask->Transparent( 'color' => 'black', 'invert' => 'True' );
	$bmask->Opaque( 'color' => 'black', fill => 'blue' );
	$opts{debug} && $bmask->Write('bmask.png');

	# Darkest image: Red pixels
	my $dmask = Image::Magick->new();
	$err = $dmask->Read("$sorted[$#sorted]");

	$dmask->Threshold( 'threshold' => "$opts{upper}%", 'channel' => 'Red' );
	$dmask->Threshold( 'threshold' => "$opts{upper}%", 'channel' => 'Green' );
	$dmask->Threshold( 'threshold' => "$opts{upper}%", 'channel' => 'Blue' );
	$opts{debug} && $dmask->Write('darkmask0.png');
	$dmask->Threshold( 'threshold' => '1%' );
	$opts{debug} && $dmask->Write('darkmask1.png');

	$dmask->Resize(
		'geometry' => "$opts{dim}x$opts{dim}",
		'filter'   => 'cubic'
	);
	$dmask->Negate();
	$dmask->Threshold( 'threshold' => '99.9%' );
	$dmask->Transparent( 'color' => 'black', 'invert' => 'True' );
	$dmask->Opaque( 'color' => 'black', fill => 'red' );
	$opts{debug} && $dmask->Write('dmask.png');

	# Use the 'greyest' image as background
	my $backgnd = Image::Magick->new;
	$err = $backgnd->Read("$median");
	$backgnd->Resize(
		'geometry' => "$opts{dim}x$opts{dim}",
		'filter'   => 'cubic'
	);

	# Create a white image of the same dimensions as the output
	my ( $width, $height ) = $backgnd->Get( 'width', 'height' );
	my $washout = Image::Magick->new;
	$washout->Set( size => "${width}x${height}" );
	$washout->ReadImage('xc:#7f7f7f');

	# Use the white image to reduce contrast and brighten the background image
	$backgnd->Composite( image => $washout, compose => 'Multiply' );
	$backgnd->Composite( image => $washout, compose => 'Add' );
	$backgnd->Composite( image => $washout, compose => 'Multiply' );
	$backgnd->Composite( image => $washout, compose => 'Add' );

	# Combine the background with the hot and cold mask
	$backgnd->Composite( image => $dmask, compose => 'SrcOver' );
	$backgnd->Composite( image => $bmask, compose => 'SrcOver' );
	$backgnd->Write("$opts{output}");
}

__END__

=head1 NAME

B<webhdr_jpgheatmap> - Produce image from an exposure-bracketed JPG sequence
indicating pixels from which reliable photometric HDR information cannot be
derived.

=head1 SYNOPSIS

webhdr_jpgheatmap [options] file1.jpg file2.jpg .. fileN.jpg

Options:
   --help           brief help message
   --man            full documentation
   --versions       print version info of loaded modules
   --debug          output intermediate image files for debugging
   -d|--dim         dimensions of output image
   -u|--upper       upper threshold
   -l|--lower       lower threshold
   -o|--output      output file name

=head1 DESCRIPTION

B<webhdr_jpgheatmap> will read an exposure-bracketed sequence of digital
photographs and analyse the pixel values in the brightest and darkest
photographs.

Pixels
which lie below the lower threshold in the brightest image or above the upper
threshold in the darkest image are shown in blue and red, resp. against a
background which is a contrast-reduced and brightness-increased version of
the photograph whose average brightness is closest to 50%.

The thresholding is done separately for the Red, Green, and Blue channels of
the source images.

B<webhdr_jpgheatmap> is meant to produce relatively small preview images. All
operations are 'infectious' in that channel-merging and downsampling will
produce the worst possible result, i.e. the highest number of hot or cold
pixels. Resizing is done with a cubic filter. This is slower than linear
sampling, but doesn't result in the artefacts that a linear operator is
prone of.

=head1 OPTIONS

=over 4

=item B<--help>

Print a brief help message and exit.

=item B<--man>

Print the manual page and exit.

=item B<--versions>

Print version information of the program, loaded modules, Perl, and the OS.

=item B<--debug>

Write out the intermediate image files for debugging. This also prints
additional information that might be helpful for troubleshooting.

=item B<-d|--dim>

Image dimensions. The output image will be resized so that its longest
dimension is equal to this value. The default for this option is 512 pixels.

=item B<-o|--output>

Output file. The default here is 'heatmap.png', which the software will
attempt to write to the working directory.

=item B<-u|--upper>

Upper threshold. This level is the range between 100 and 255, and
marks the bright pixels
on the darkest photograph above which reliable photometric information
cannot be determined. The default for this is 228.
Pixels above this value will be shown as 'hot' pixels.
The thresholds used to be 20 (upper) and 200 (lower, see below)
See http://www.radiance-online.org/pipermail/hdri/2012-February/000378.html

=item B<-l|--lower>

Lower threshold. This is the level of the darkest pixels
on the brightest photograph below which reliable photometric information
cannot be determined. The default for this is 27.
Pixels below this value will be shown as 'cold' pixels.

=back

=head1 EXAMPLE

To generate an image whose largest dimension is 400 px and output the result
to a file named hotcold.png:

  webhdr_jpgheatmap -o hotcold.png --dim 400 sequences/DSCN286?.JPG

=head1 PREREQUISITES

Perl bindings for ImageMagick (http://www.imagemagick.org/script/perl-magick.php/)

=head1 ENVIRONMENT

webhdrtools's lib directory needs to be in PERL5LIB

=head1 LIMITATIONS

=over 4

=item * The result is always stored in PNG format

=item * The colours for the hot and cold pixels are always red and blue, resp.

=item * The way the background image is selected and processed is hard-coded.

=back

=head1 COPYRIGHT

Copyright 2013 Axel Jacobs

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

#######################################################################
# ChangeLog
# ---------
#
# 0.27,   02 Jun 2013: Minor typos
# 0.26,   05 Aug 2012: Minor corrections in POD
# 0.25,   20 Jan 2012: Ran perltidy. See README_tidy.txt
# 0.24,   20 Jan 2012: Implemented short option names.
# 0.23,   20 Jan 2012: Renamed to webhdr_jpgheatmap.pl
# 0.22,   17 May 2011: Thresholding is now applied to indidual channels
#                      rather than separate layer images.
# 0.21,   17 May 2011: Changed name of main sub to hdrheatmap.
# 0.20,   17 May 2011: Changed default output file to heatmap.png
# 0.15,   16 Apr 2011: Undid 0.14
# 0.14,   16 Apr 2011: Renamed main sub to 'main'
# 0.13,   16 Apr 2011: Set getopt to no_auto_abbre, removed some long opts
# 0.12,   16 Apr 2011: Processing hot after cold now, not at the same time
#                      to reduce memory consumption
# 0.11,   12 Apr 2011: Renamed to hdrheatmap.pl
# 0.10,   30 Mar 2011: Initial version
#######################################################################

#EOF
