#!/usr/bin/perl

use strict;
use warnings;
use Pod::Usage;
use Getopt::Long;

use Exif;
use Exposure;

our $VERSION = '0.13';

my $help     = 0;
my $man      = 0;
my $versions = 0;

my %options = (
	'help'     => \$help,
	'man'      => \$man,
	'versions' => \$versions,
);

GetOptions( \%options, 'help', 'man', 'versions', )
  or pod2usage( -verbose => 0 ) && exit 1;

if ($versions) {
	print
	  "\nModules, Perl, OS, Program info:\n",
	  "  Pod::Usage            $Pod::Usage::VERSION\n",
	  "  Getopt::Long          $Getopt::Long::VERSION\n",
	  "  strict                $strict::VERSION\n",
	  "  warnings              $warnings::VERSION\n",
	  "  Perl                  $]\n",
	  "  OS                    $^O\n",
	  "  jpginfo               $VERSION\n",
	  "  Exif                  $Exif::VERSION\n",
	  "  Exposure              $Exposure::VERSION\n\n";
	exit 0;
}

pod2usage( -verbose => 1 ) && exit if $help;
pod2usage( -verbose => 2 ) && exit if $man;
pod2usage("$0: No files given.\n") && exit
  unless ( ( $#ARGV >= 0 ) or caller() or $options{undo} );
foreach (@ARGV) {
	push( @{ $options{filenames} }, $_ );    # JPEG images
}

my $err = webhdr_jpginfo(%options) unless caller();
if ($err) { die("$0: $err\n"); }

sub webhdr_jpginfo {
	my %opts = @_;

	foreach ( @{ $options{filenames} } ) {
		my $exif     = &get_exif_tags($_);
		my $shutter  = &get_shutter($exif);
		my $aperture = &get_aperture($exif);
		my $iso      = &get_iso($exif);
		my $expo     = &exposure_from_exif($exif);
		$expo = sprintf( "%.2f", $expo );

		print "$_\tISO$iso\tF$aperture\t${shutter}s\t${expo}EV\n";
	}
	return;
}

__END__

=head1 NAME

B<webhdr_jpginfo> - Extract exposure information from the EXIF header

=head1 SYNOPSIS

webhdr_jpginfo file1.jpg file2.jpg .. fileN.jpg

Options:
   -help           brief help message
   -man            full documentation
   -versions       print version info of loaded modules

=head1 DESCRIPTION

B<webhdr_jpginfo> display exposure-relevant information for the given
digital photographs, one per line. The output is tab-separated text. The
following output is produced:

=over 2

=item * filename

=item * ISO

=item * FNumber

=item * ShutterSpeed

=item * Exposure

=back

=head1 OPTIONS

=over 4

=item B<-help>

Print a brief help message and exit.

=item B<-man>

Print the manual page and exit.

=item B<-versions>

Print version information of the program, loaded modules, Perl, and the OS.

=back

=head1 EXAMPLE

Print exposure information for a whole directory of photographs:

  webhdr_jpginfo holidays1/DSC_*.JPG

=head1 PREREQUISITES

Phil Harvey's ExifTool (http://www.sno.phy.queensu.ca/~phil/exiftool/)

=head1 ENVIRONMENT

webhdrtools's lib directory needs to be in PERL5LIB

=head1 COPYRIGHT

Copyright 2013 Axel Jacobs

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

#######################################################################
# ChangeLog
# ---------
#
# 0.13,   02 Jun 2012: Upped (c) year
# 0.12,   12 Aug 2012: POD fixes
# 0.11,   05 Aug 2012: Minor fixes
# 0.10,   28 Apr 2012: Initial version: --info option from webhdr_jpgsep.pl
#######################################################################

#EOF
