#!/usr/bin/perl

use strict;
use warnings;
use Pod::Usage;
use Getopt::Long;
use Time::Local;
use File::Basename;

use Exif;
use Exposure;

our $VERSION = '0.13';

my $help     = 0;
my $man      = 0;
my $versions = 0;
my $debug    = 0;

my %options = (
	'debug'    => 0,
	'help'     => \$help,
	'man'      => \$man,
	'versions' => \$versions,
	'gap'      => 30,
	'base'     => 'hdr',
	'yes'      => 0,
	'no'       => 0,
	'undo'     => 0,
	'verbose'  => 0,
);

GetOptions(
	\%options,  'debug', 'help', 'man',    'versions', 'gap|g=i',
	'base|b=s', 'yes|y', 'no|n', 'undo|u', 'verbose',
) or pod2usage( -verbose => 0 ) && exit 1;

if ($versions) {
	print
	  "\nModules, Perl, OS, Program info:\n",
	  "  Pod::Usage            $Pod::Usage::VERSION\n",
	  "  Getopt::Long          $Getopt::Long::VERSION\n",
	  "  Time::Local           $Time::Local::VERSION\n",
	  "  File::Basename        $File::Basename::VERSION\n",
	  "  strict                $strict::VERSION\n",
	  "  warnings              $warnings::VERSION\n",
	  "  Perl                  $]\n",
	  "  OS                    $^O\n",
	  "  jpgsep                $VERSION\n",
	  "  Exif                  $Exif::VERSION\n",
	  "  Exposure              $Exposure::VERSION\n\n";
	exit 0;
}

pod2usage( -verbose => 1 ) && exit if $help;
pod2usage( -verbose => 2 ) && exit if $man;
pod2usage("$0: No files given.\n") && exit
  unless ( ( $#ARGV >= 0 ) or caller() or $options{undo} );
foreach (@ARGV) {
	push( @{ $options{filenames} }, $_ );    # JPEG images
}

# Make sure output is verbose if debug is requested.
if ( $options{debug} ) { $options{verbose} = 1; }

webhdr_jpgsep(%options) unless caller();

sub webhdr_jpgsep {
	my %opts     = @_;
	my @images   = ();
	my @capdates = ();
	my @shutters = ();
	my @subdirs  = ();

	# Check if any directories $basename\d\d exist.
	opendir( DH, "." );
	while ( readdir(DH) ) {
		next unless -d;
		if (m/^$opts{base}/) {
			push( @subdirs, $_ );
		}
	}
	closedir(DH);
	if ( $opts{undo} ) {
		&move_from_subdirs(@subdirs);
	} elsif ( $#subdirs + 1 ) {
		die("directory with requested name already exists: $subdirs[0]");
	}

	foreach ( @{ $options{filenames} } ) {
		push( @images, $_ );

		my $exif    = &get_exif_tags($_);
		my $dstring = $$exif{CreateDate};

		# e.g. 2012:02:23 13:13:16
		$dstring =~ /(\d+):(\d+):(\d+) (\d+):(\d+):(\d+)/;
		my $year   = $1;
		my $month  = $2;
		my $day    = $3;
		my $hour   = $4;
		my $minute = $5;
		my $second = $6;
		push( @capdates,
			timelocal( $second, $minute, $hour, $day, $month, $year ) );

		my %settings = &get_settings($exif);
		my $shutter  = &get_shutter($exif);
		push( @shutters, $shutter );
	}

	# A 'gap' is the time difference between THIS frame and the NEXT frame,
	# minus the exposure time of the NEXT frame.
	# Hence, @gaps is one index shorter than @shutters or @images.
	my @gaps = ();
	for ( my $i = 0 ; $i < $#capdates ; $i++ ) {

		# The recorded capture time is at the END of the exposure period!
		my $gap    = $capdates[ $i + 1 ] - $capdates[$i] - $shutters[ $i + 1 ];
		my $intgap = int( $gap + .5 );

		# Only deal with inter-frame gaps as integers (full seconds)
		push( @gaps, $intgap );
		$opts{debug} && print "$images[$i] ($shutters[$i]s) : $capdates[$i]\n";
		$opts{debug} && print "$intgap\n";
	}
	$opts{debug}
	  && print
	  "$images[$#capdates] ($shutters[$#capdates]s) : $capdates[$#capdates]\n";

	my @sequences = ( [] );
	my @framegaps = ( [] );
	my $seqid     = 0;
	my @seqgaps   = ();
	for my $i ( 0 .. $#gaps ) {
		push( $sequences[$seqid], $images[$i] );
		push( $framegaps[$seqid], $gaps[$i] );
		if ( $gaps[$i] >= $opts{gap} ) {
			push( @sequences, [] );
			push( @framegaps, [] );
			$seqid++;
			push( @seqgaps, $gaps[$i] );
		}
	}

	# The very last image has no 'gap'.
	push( $sequences[$seqid], $images[$#images] );

	my $counter = 0;
	for my $i ( 0 .. $#sequences ) {
		my @sequence = @{ $sequences[$i] };

		#print "#$i ($#sequence frames): [ @{$sequences[$i]} ],\n";
		my $nframes = $#sequence + 1;
		my $dirname = sprintf( "$opts{base}%02u", $i + 1 );
		$opts{verbose} && print "#$i ($nframes frames => $dirname/):\n";
		for my $j ( 0 .. $#sequence ) {
			$opts{verbose}
			  && print "\t$sequence[$j] -> $shutters[$counter] sec\n";
			$counter++;
		}
	}
	$opts{verbose} && print "Requested gap time: $opts{gap} seconds\n";
	$counter = 0;
	for my $i ( 0 .. $#seqgaps ) {
		my @sequence = @{ $sequences[$i] };
		my $seqlen   = $#sequence + 1;
		print "[#$counter ($seqlen)] $seqgaps[$i]s ";
		$counter++;
	}
	my @sequence = @{ $sequences[$#sequences] };
	my $seqlen   = $#sequence + 1;
	print "[#$counter ($seqlen)]\n";

	my $yes = $opts{yes};
	my $no  = $opts{no};
	if ($no) {
		$yes = 0;
	} elsif ( !$yes ) {
		print "Do you wish to proceed and move the files [y|N]?";
		chomp( my $input = <STDIN> );
		if ( $input =~ /^[Yy]/ ) { $yes = 1; }
	}
	if ($yes) {
		$opts{verbose} && print "Moving files...\n";
		for my $i ( 0 .. $#sequences ) {
			my @sequence = @{ $sequences[$i] };
			my $dirname = sprintf( "$opts{base}%02u", $i + 1 );
			mkdir $dirname or die("directory exists: $dirname\n");
			for my $j ( 0 .. $#sequence ) {
				my $path = $sequence[$j];
				my ( $filename, $directories, $suffix ) = fileparse($path);
				rename( $path, "$dirname/$filename" )
				  or die("Unable to rename $path to $dirname/$filename: $!");
			}
		}
	}
	return;
}

sub count_unique_gaps {
	my @array = @_;
	my %cnt;
	map { $cnt{$_}++ } @array;

	return %cnt;
}

sub move_from_subdirs {

	# See if any of the directories contains a sub-directory or non-JPEG files.
	foreach my $i (@_) {
		opendir( DH, "$i" );
		while ( readdir(DH) ) {
			if ( -d "$i/$_" ) {
				next if ( (m/^\.\.$/) or (m/^\.$/) );
				die( "Cannot undo: directory '$i' contains a sub-directory '$_'"
				);
			} elsif ( !m/JPE?G$/i ) {
				die( "Cannot undo: directory '$i' contains an unknown file '$_'"
				);
			} elsif ( -f $_ ) {
				die(
					"Cannot undo: file '$_' from directory '$i' exsists in pwd."
				);
			}
		}
		close DH;
	}

	# Do the actual move.
	foreach my $i (@_) {
		opendir( DH, "$i" );
		while ( readdir(DH) ) {
			next if ( (m/^\.\.$/) or (m/^\.$/) );
			rename( "$i/$_", $_ ) or die("Unable to rename: $?");
		}
		close DH;
		rmdir $i;
	}
}

__END__

=head1 NAME

B<webhdr_jpgsep> - Separate sequences of exposure-bracketed images into their
own sub-directories.

=head1 SYNOPSIS

webhdr_jpgsep [options] file1.jpg file2.jpg .. fileN.jpg

Options:
   -help           brief help message
   -man            full documentation
   -versions       print version info of loaded modules
   -debug          output intermediate image files for debugging
   -g|--gap        time between two successive frames
   -b|--base       base name of the output directory
   -y|--yes        skip confirmation
   -u|--undo       undo: move JPEGs out of their sub-directories
   --verbose       verbose: produce additional output.

=head1 DESCRIPTION

B<webhdr_jpgsep> will separate all JPEG images given to it as arguments into
individual sub-directories. The criterion for this is the inter-frame gap,
which is the time difference between the capture date of a frame and the next
frame. The images need to be passed in the correct order for this to work.
Shell wild cards seem to work well, while Perl directory listings DO NOT.
The algorithm is smart enough to subtract the time it too to shoot the next
frame from the gap time, although this is obviously only any good for
exposure times beyond 1 second. The default gap is 30 seconds. This can be
over-written with the B<-g gap> option.

Directories will be named with B<-b basename>, followed by a running index
that is left-padded with a zero to two places, e.g. 01. If any directory
with a name starting with B<basename> exists, the program will drop out.
The default for B<-b> is 'hdr'.

After looking at all images and determining their gaps, B<webhdr_jpgsep>
produces detailed output about the action that it is about to take. This
needs to be confirmed by the user, which requires the input of 'y', followed
by a return. This interactive confirmation can be disabled with the B<-y>
option. It is highly recommended to set the B<--verbose> switch for additional
output.

The B<-u> option can undo the process: It will move all JPEGs out of their
sub-directories into the working directory. For this, it needs to know which
directories to process, which is again given with the B<-b base> option.

B<webhdr_jpgsep> will refuse to work with the B<-u> option if any of the
directories contain sub-directories, if they contain files other than JPEG
images, or if the files that are to be moved already exist in the current
working directory.

=head1 OPTIONS

=over 4

=item B<-help>

Print a brief help message and exit.

=item B<-man>

Print the manual page and exit.

=item B<-versions>

Print version information of the program, loaded modules, Perl, and the OS.

=item B<-debug>

Write out the intermediate image files for debugging. This also prints
additional information that might be helpful for troubleshooting.

=item B<-g|--gap>

Gap between two frames, in seconds. The default for this option is 30
seconds. The time it took to take the next frame is considered in the
calculation.

=item B<-b|--base>

Base name of the new directories. The default is 'hdr', resulting in
directories 'hdr01', 'hdr02' etc.

=item B<-y|--yes>

Skip the interactive confirmation (answer 'yes').

=item B<-u|--undo>

Undo: Move all JPEG images from the directories whose names starts with
B<-b base> back to the working directory.

=item B<--verbose>

Print additional information about the images in each sequence and their
exposure times before confirmation is requested.

=back

=head1 EXAMPLE

Separate JPGs into sequences that are at least 40 seconds apart:

  webhdr_jpgsep -g 40 -b laboratory_hdr DSC_*.JPG

Undo the previous command and remove all directories, which are now empty.

  webhdr_jpgsep -b laboratory_hdr -u

=head1 LIMITATIONS

=over 4

=item * The images need to be passed in the correct order.

=back

=head1 COPYRIGHT

Copyright 2013 Axel Jacobs

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

#######################################################################
# ChangeLog
# ---------
#
# 0.12,   02 Jun 2013: Upped (c) year
# 0.11,   28 Apr 2012: Removed --info option: New webhdr_jpginfo.pl
# 0.10,   26 Feb 2012: Initial version
#######################################################################

#EOF
