#!/usr/bin/perl

our $VERSION = '0.22';

use warnings;
use strict;
use Pod::Usage;
use Getopt::Long;

use RSP;

my $help     = 0;
my $man      = 0;
my $versions = 0;

my %options = (
	'help'     => \$help,
	'man'      => \$man,
	'debug'    => 0,
	'versions' => \$versions,
	'output'   => '-',
);

GetOptions( \%options, 'debug', 'help', 'man', 'versions', 'output|o=s', )
  or pod2usage( -verbose => 0 ) && exit 1;

if ( $versions ) {
	print
	  "\nModules, Perl, OS, Program info:\n",
	  "  Pod::Usage            $Pod::Usage::VERSION\n",
	  "  Getopt::Long          $Getopt::Long::VERSION\n",
	  "  strict                $strict::VERSION\n",
	  "  warnings              $warnings::VERSION\n",
	  "  Perl                  $]\n",
	  "  OS                    $^O\n",
	  "  rspavrg               $VERSION\n",
	  "  RSP                   $RSP::VERSION\n\n";
	exit;
}

pod2usage( -verbose => 1 ) && exit if $help;
pod2usage( -verbose => 2 ) && exit if $man;
pod2usage("$0: Need at least two RSP files.\n") && exit unless ( $#ARGV >= 0 );
foreach (@ARGV) {
	push( @{ $options{rspfiles} }, $_ );
}

avrgrsp(%options) unless caller();

sub avrgrsp {
	my %opts  = @_;
	my @files = ();
	foreach ( @{ $opts{rspfiles} } ) {
		die("Can't find RSP file $_\n") unless (-f);

		#if (my $validation_error = &rsp_validate( $_ )) {
		#die ("Invalid file: $validation_error\n");
		#}
		push( @files, $_ );
	}

	my @channels = ( [], [], [] );
	foreach my $rsp (@files) {
		$opts{debug} && print "file: $rsp\n";
		open RSPFH, "$rsp" or die $!;
		for my $channel (@channels) {
			my $line = readline(RSPFH);
			$opts{debug} && print "$line";
			chomp $line;
			push( $channel, $line );
		}
		close RSPFH;
	}

	open FH, ">$opts{output}"
	  or die("Can't open rsp file $opts{output} for writing: $!\n");
	for my $channel (@channels) {
		my $avrg = &rsp_average($channel);
		print FH "$avrg\n";
	}
	close FH;

	# Unique is not yet working...
	#if ( $opts{unique} ) {
	#}
}

__END__

=head1 NAME

webhdr_avrgrsp - Average the given RSP polynomials.

=head1 SYNOPSIS

webhdr_rspavrg [options] 1.rsp 2.rsp [3.rsp .. N.rsp]

Options:
  --help           brief help message
  --man            full documentation
  --versions       print version info of loaded modules
  --debug          output intermediate image files for debugging
  -o               output file

=head1 DESCRIPTION

B<webhdr_avrgrsp> takes the average of the RSP polynomials.

=head1 OPTIONS

=over 4

=item B<-h|--help>

Print a brief help message and exit.

=item B<-m|--man>

Print the manual page and exit.

=item B<-v|--versions>

Print version information of the program, loaded modules, Perl, and the OS.

=item B<--debug>

Write out the intermediate image files for debugging. This also prints
additional information that might be helpful for troubleshooting.

=item B<-o|--output>

Output file. The default is output to STDOUT.

=back

=head1 EXAMPLE

To take the average over three RSP files and output to STDOUT:

  webhdr_rspavrg.pl sequence1.rsp sequence2.rsp sequence3.rsp

=head1 COPYRIGHT

Copyright 2013 Axel Jacobs

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

#######################################################################
# ChangeLog
# ---------
#
# 0.22,  02 Jun 2013: Upped (c) year
# 0.21,  12 Aug 2012: Output our VERSION with -versions.
# 0.1,   01 Apr 2011: Initial version
#######################################################################

#EOF
