#!/usr/bin/perl

our $VERSION = '0.23';

use warnings;
use strict;
use Pod::Usage;
use Getopt::Long;
use File::Temp qw/ tempfile /;

use RSP;

my $help     = 0;
my $man      = 0;
my $versions = 0;

my %options = (
	'debug'    => 0,
	'help'     => \$help,
	'man'      => \$man,
	'versions' => \$versions,
	'output'   => 'rsp.png',
	'crop'     => 0,
	'title'    => '',
	'labels'   => 0,
	'tics'     => 0,
	'xdim'     => 400,
	'ydim'     => 300,
	'inputs'   => [],
);

# For WebHDR: 'xdim' => 384, 'ydim' => 288,
GetOptions(
	\%options, 'debug',    'help', 'man',    'versions', 'xdim=i',
	'ydim=i',  'output=s', 'crop', 'labels', 'tics',     'title=s',
) or pod2usage( -verbose => 0 ) && exit 1;

if ($versions) {
	print
	  "\nModules, Perl, OS, Program info:\n",
	  "  Pod::Usage            $Pod::Usage::VERSION\n",
	  "  Getopt::Long          $Getopt::Long::VERSION\n",
	  "  File::Temp            $File::Temp::VERSION\n",
	  "  strict                $strict::VERSION\n",
	  "  warnings              $warnings::VERSION\n",
	  "  Perl                  $]\n",
	  "  OS                    $^O\n",
	  "  rspplot               $VERSION\n",
	  "  RSP                   $RSP::VERSION\n\n";
	exit;
}

pod2usage( -verbose => 1 ) && exit if $help;
pod2usage( -verbose => 2 ) && exit if $man;
pod2usage("$0: Need at least on RSP file.\n") && exit
  unless ( ( $#ARGV >= 0 ) or caller() );
foreach (@ARGV) {
	push( @{ $options{inputs} }, $_ );    # RSP files
}

plotrsp(%options) unless caller();

sub plotrsp {
	my %opts = @_;

	if ( $opts{debug} ) {
		print("Running in debug mode...\n");
		while ( my ( $key, $value ) = each(%opts) ) {
			print "$key => $value\n";
		}
		print "\n";
	}

	my @rsp = ();
	foreach ( @{ $options{inputs} } ) {
		my $file = $_;
		die("Can't find RSP file $file\n") unless ( -f "$file" );
		if ( my $validation_error = &rsp_validate("$file") ) {
			die("Invalid file: $validation_error\n");
		}
		push( @rsp, "$file" );
	}

	open RSPFH, "$rsp[0]" or die $!;
	my $redline = readline(RSPFH);
	my $grnline = readline(RSPFH);
	my $bluline = readline(RSPFH);
	close(RSPFH);

	my $redstr = &rsp_plotstring("$redline");
	my $grnstr = &rsp_plotstring("$grnline");
	my $blustr = &rsp_plotstring("$bluline");

	my $plotstring = &merge_plotstrings( $redstr, $grnstr, $blustr );

	my $crop = '';
	if ( $opts{crop} ) {
		$crop = 'crop';
	}
	my $labels = '';
	if ( $opts{labels} ) {
		$labels = "set xlabel 'Pixel value'\nset ylabel 'Brightness'";
	}
	my $tics = "set notics";
	if ( $opts{tics} ) {
		$tics = '';
	}

	# Write the gnuplot file
	my ( $plotfh, $plotfile ) = tempfile( SUFFIX => '.gnuplot' );

	print $plotfh <<EndOfPlot;
set title "$opts{title}"
set terminal png $crop size $opts{xdim},$opts{ydim} small
set output "$opts{output}"
set xrange [0:1]
set yrange [0:1]
$labels
$tics
unset key
$plotstring
EndOfPlot
	close $plotfh;

	system( 'gnuplot', "$plotfile" );

	if ( $opts{debug} ) {
		print "Keeping gnuplot file: $plotfile\n";
	} else {
		unlink($plotfile);
	}

}

__END__

=head1 NAME

B<webhdr_rspplot> - Plot the hdrgen response curve to a PNG image

=head1 SYNOPSIS

webhdr_rspplot [options] 1.rsp [2.rsp .. N.rsp]

Options:
   --help           brief help message
   --man            full documentation
   --versions       print version info of loaded modules
   --debug          output intermediate image files for debugging
   --xdim           width of graph
   --ydim           height of graph
   --title          graph title
   --crop           crop white padding around graph
   --label          label x and y axes
   --tics           include axis tics with labels
   --output         output file name

=head1 DESCRIPTION

B<webhdr_rspplot> will read one or more RSP files containing the response curve
coefficients for/from processing an exposure-bracketed sequence of photographs
into an HDR image. Those coefficient define a polynomical which rspplot will
plot to a PNG image with the help of gnuplot.

=head1 OPTIONS

=over 4

=item B<--help>

Print a brief help message and exit.

=item B<--man>

Print the manual page and exit.

=item B<--versions>

Print version information of the program, loaded modules, Perl, and the OS.

=item B<--debug>

Keep the gnuplot file for debugging. This also prints
additional information that might be helpful for troubleshooting.

=item B<--xdim, --ydim>

The width and height of the graph. The default for these options is
400 and 300 pixels, resp.

=item B<--output>

Output file. The default here is 'rsp.png', which the software will attempt
to write to the working directory.

=item B<--crop>

Crop the white border around the graph, labels and title. This is not terribly
useful because the final dimensions of the graph are difficult to predict.

=item B<--labels>

Label the x and y axes. If this option is enabled, the x-axis will be labelled
'Pixel value', and the y-axis 'Brightness'. This is not configurable.

=item B<--title>

Title of the graph. If not given, not title will be put on the graph.

=back

=head1 PREREQUISITES

rspplot relies on gnuplot which must be installed and be found in $PATH.

=head1 EXAMPLE

To plot the response curve for file snapshot.rsp, giving it the title
"Snapshot Box" and saving the PNG to snapshot_reponse.png:

  rspplot.pl --labels -x 384 -y 288 -o snapshot_response.png --title
  "Snapshot Box" snapshot.rsp

=head1 BUGS

Currently, only the first RSP file is processed. Any additional ones are
ignored. If more than one RSP file is given, the green channel (option?)
should be plotted for all...

=head1 COPYRIGHT

Copyright 2013 Axel Jacobs

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut

#######################################################################
# ChangeLog
# ---------
#
# 0.22,   02 Jun 2013: Minor typos
# 0.21,   17 May 2011: Updated POD
# 0.20,   17 May 2011: Args are now parsed into a hash
# 0.11,   01 Apr 2011: no_auto_abbrev; removed some long options
# 0.10,   01 Apr 2011: Initial version
#######################################################################

#EOF
